/*
 * Simulator of MCS51 (brk.cc)
 *
 * Copyright (C) 1997,97 Drotos Daniel, Talker Bt.
 * 
 * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
 *
 */

/* This file is part of MCS51 simulator: s51.

S51 is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

S51 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with S51; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */
/*@1@*/

#include "ddconfig.h"

#include "pobjcl.h"
#include "brkcl.h"


/*
 * Base object of breakpoints
 *____________________________________________________________________________
 *
 */

cl_brk::cl_brk(uint iaddr,
	       enum brk_perm iperm,
	       int  ihit):
  cl_base()
{
  addr = iaddr;
  perm = iperm;
  hit  = ihit;
  cnt  = ihit;
}

cl_brk::~cl_brk(void)
{}


/*
 * FETCH type of breakpoint
 *____________________________________________________________________________
 *
 */

cl_fetch_brk::cl_fetch_brk(uint iaddr,
			   enum brk_perm iperm,
			   int  ihit):
  cl_brk(iaddr, iperm, ihit)
{
  code = 0;
}

enum brk_type
cl_fetch_brk::type(void)
{
  return(brkFETCH);
}


/*
 * Base of EVENT type of breakpoints
 *____________________________________________________________________________
 *
 */

cl_ev_brk::cl_ev_brk(uint iaddr,
		     enum brk_perm iperm,
		     int  ihit,
		     enum brk_event ievent,
		     const char *iid):
  cl_brk(iaddr, iperm, ihit)
{
  event= ievent;
  id   = iid;
}

enum brk_type
cl_ev_brk::type(void)
{
  return(brkEVENT);
}

bool
cl_ev_brk::match(struct event_rec *ev)
{
  return(FALSE);
}


/*
 * WRITE IRAM type of EVENT breakpoints
 *____________________________________________________________________________
 *
 */

cl_wi_brk::cl_wi_brk(uint iaddr,
		     enum brk_perm iperm,
		     int  ihit):
  cl_ev_brk(iaddr, iperm, ihit, brkWIRAM, "wi")
{
}

bool
cl_wi_brk::match(struct event_rec *ev)
{
  return(ev->wi == addr);
}


/*
 * READ IRAM type of EVENT breakpoints
 *____________________________________________________________________________
 *
 */

cl_ri_brk::cl_ri_brk(uint iaddr,
		     enum brk_perm iperm,
		     int  ihit):
  cl_ev_brk(iaddr, iperm, ihit, brkRIRAM, "ri")
{
}

bool
cl_ri_brk::match(struct event_rec *ev)
{
  return(ev->ri == addr);
}


/*
 * WRITE XRAM type of EVENT breakpoints
 *____________________________________________________________________________
 *
 */

cl_wx_brk::cl_wx_brk(uint iaddr,
		     enum brk_perm iperm,
		     int  ihit):
  cl_ev_brk(iaddr, iperm, ihit, brkWXRAM, "wx")
{
}

bool
cl_wx_brk::match(struct event_rec *ev)
{
  return(ev->wx == addr);
}


/*
 * READ XRAM type of EVENT breakpoints
 *____________________________________________________________________________
 *
 */

cl_rx_brk::cl_rx_brk(uint iaddr,
		     enum brk_perm iperm,
		     int  ihit):
  cl_ev_brk(iaddr, iperm, ihit, brkRXRAM, "rx")
{
}

bool
cl_rx_brk::match(struct event_rec *ev)
{
  return(ev->rx == addr);
}


/*
 * WRITE SFR type of EVENT breakpoints
 *____________________________________________________________________________
 *
 */

cl_ws_brk::cl_ws_brk(uint iaddr,
		     enum brk_perm iperm,
		     int  ihit):
  cl_ev_brk(iaddr, iperm, ihit, brkWSFR, "ws")
{
}

bool
cl_ws_brk::match(struct event_rec *ev)
{
  return(ev->ws == addr);
}


/*
 * READ SFR type of EVENT breakpoints
 *____________________________________________________________________________
 *
 */

cl_rs_brk::cl_rs_brk(uint iaddr,
		     enum brk_perm iperm,
		     int  ihit):
  cl_ev_brk(iaddr, iperm, ihit, brkRSFR, "rs")
{
}

bool
cl_rs_brk::match(struct event_rec *ev)
{
  return(ev->rs == addr);
}


/*
 * READ CODE type of EVENT breakpoints
 *____________________________________________________________________________
 *
 */

cl_rc_brk::cl_rc_brk(uint iaddr,
		     enum brk_perm iperm,
		     int  ihit):
  cl_ev_brk(iaddr, iperm, ihit, brkRCODE, "rc")
{
}

bool
cl_rc_brk::match(struct event_rec *ev)
{
  return(ev->rc == addr);
}


/*
 * Making collection of break-points
 *____________________________________________________________________________
 *
 */

brk_coll::brk_coll(t_index alimit, t_index adelta):
  cl_sorted_list(alimit, adelta)
{
}


/*
 * Collection of break-points
 *____________________________________________________________________________
 *
 * This is a sorted collection, sorted by addr field of brk items.
 *
 */

void *
brk_coll::key_of(void *item)
{
  return((void *)&(((class cl_brk *)(item))->addr));
}


/*
 * Comparing key fields of two brk items
 *____________________________________________________________________________
 *
 */

int
brk_coll::compare(void *key1, void *key2)
{
  int k1, k2;

  k1= *(int *)key1;
  k2= *(int *)key2;

  if (k1 == k2)
    return(0);
  else
    if (k1 < k2)
      return(-1);
    else
      return(+1);
}


/*
 * Checking if there is an event breakpoint for the specified event
 *____________________________________________________________________________
 *
 */

bool
brk_coll::there_is_event(enum brk_event ev)
{
  class cl_brk *b;
  int   i;

  for (i= 0; i < count; i++)
    {
      b= (class cl_brk *)at(i);
      if (b->type() == brkEVENT &&
	  ((class cl_ev_brk *)b)->event == ev)
	return(TRUE);
    }
  return(FALSE);
}


/* End of brk.cc */
